import pandas as pd
import os
import sklearn.preprocessing
import pickle
import numpy as np
import os
import imblearn
def data_dictionary(data, path=''):
    """
    Function creates the data dictionary csv for a given dataset

    args:
    ------------
        - data [Pandas.DataFrame object] - dataset table to create the
                                            dictionary from
        - path [string] - location where to save the dictionary
    """
    cols = data.columns

    DD = pd.DataFrame(columns = ['Feature', 'Type', 'Description', 'Mean',
                                 'Median', 'Std'])
    DD['Feature'] = cols
    DD['Type'] = pd.Series([ str(data[column].dtype) for column in cols])
    DD = DD.set_index(DD['Feature']).drop('Feature', axis = 1)
    DD.to_csv(path + '\\data_dict.csv')


def text_columns(data, name = 'features'):
    with open(os.getcwd()+'//' + name + '.txt', 'w') as f:
        f.write('Dataset features \n -------------------- \n')
        for name in data.columns:
            f.write(name +', ' + str(data[name].dtype) + '\n')


def to_classes(Ys, n_classes, division = 'median', perc = None):

    if division == 'percentile':
        classes_ys = percentile_division(Ys, n_classes, perc)

    elif  division == 'median':
        if n_classes == 2:
            classes_ys = matrix_to_binary(Ys)

        if n_classes == 4:
            classes_ys = matrix_to_four_classes(Ys)
    else:
        raise ValueError('Unknown division type')

    classes_ys = classes_ys
    return classes_ys

def upsample(x, y):
    upsm = imblearn.over_sampling.SMOTE(random_state=42)
    x, y =  upsm.fit_resample(x,y)
    return x,y


def percentile_division(Ys, n_classes, perc):
    assert len(perc) == n_classes-1, 'Wrong number of percentile edges'
    Y = Ys.copy()
    for name in Y.columns:
        init = Ys[name].min()-1
        class_ = 0.0
        for p in perc:
            val = np.percentile(Ys[name], p)
            mask1 = Ys[name] >= init
            mask2 = Ys[name] < val
            Y.loc[mask1 & mask2, name] = class_
            init = val
            class_ += 1
        final_mask = Ys[name] >= init
        Y.loc[final_mask, name] = class_

    return Y

def matrix_to_binary(Ys):
    binary = pd.DataFrame(np.zeros(Ys.shape), columns = Ys.columns)

    for name in Ys.columns:

        half = np.median(Ys[name])
        binary.loc[Ys[name] <= half, name] = 0
        binary.loc[Ys[name] > half, name] = 1
        binary[name] = binary[name].astype('int32')


    return binary

def matrix_to_four_classes(Y):

    quarter = pd.DataFrame(np.zeros(Y.shape), columns = Y.columns)

    for name in Y.columns:
        half = np.median(Y[name])
        q1 = np.median(Y[Y[name] <= half])
        q3 = np.median(Y[Y[name] > half])
        quarter.loc[Y[name]<=q1, name] = 0

        mask1 = Y[name]>q1
        mask2 = Y[name]<=half
        quarter.loc[(mask1 & mask2), name] = 1

        mask3 = Y[name] > half
        mask4 = Y[name] <= q3
        quarter.loc[(mask3 & mask4) , name] = 2

        quarter.loc[Y[name] > q3, name] = 3

    quarter[name] = quarter[name].astype('int32')

    return quarter


def delete_unique(data, exceptions = []):
    to_drop = []
    for name in data.columns:
        if (data[name].unique().shape[0] == data.shape[0] or \
          data[name].unique().shape[0] == 1) and name not in exceptions:
            to_drop.append(name)

    data.drop(to_drop, axis = 1, inplace = True)

    return data, to_drop


def label_encode(data, features):

    label_dict = {}

    for feature in features:
        label_dict['enc_{}'.format(feature)] = \
            sklearn.preprocessing.LabelEncoder()
        temp_feature = \
            label_dict['enc_{}'.format(feature)].fit_transform(data[feature])
        data[feature] = temp_feature



    return data, label_dict


def scale(data, kind='minmax', ignore=None):
    if kind == 'minmax':
        scaler = sklearn.preprocessing.MinMaxScaler()
    elif kind == 'standard':
        scaler = sklearn.preprocessing.StandardScaler()
    if ignore is not None:
        assert type(ignore) == list or tuple
        temp = data[ignore]
        data = data.drop(ignore, axis = 1)


    if type(data) == pd.DataFrame:
        cols = data.columns
        data = scaler.fit_transform(data)
        data = pd.DataFrame(data, columns = cols)
    else:
        data = scaler.fit_transform(data)


    data = pd.concat((data.reset_index(drop=True),
                      temp.reset_index(drop=True)),
                      axis = 1)

    return data


    if __name__ == '__main__':
        pass
